"""
Created on Jan 10, 2014

@author: sean
"""
from __future__ import print_function, unicode_literals, absolute_import
import sys
import struct
from pprint import pprint
from .errors import RPMError

tags = {
    "signature": 267,
    "md5": 269,
    "name": 1000,
    "version": 1001,
    "release": 1002,
    "serial": 1003,
    "summary": 1004,
    "description": 1005,
    "buildtime": 1006,
    "buildhost": 1007,
    "installtime": 1008,
    "size": 1009,
    "distribution": 1010,
    "vendor": 1011,
    "gif": 1012,
    "xpm": 1013,
    "copyright": 1014,
    "packager": 1015,
    "group": 1016,
    "changelog": 1017,
    "source": 1018,
    "patch": 1019,
    "url": 1020,
    "os": 1021,
    "arch": 1022,
    "prein": 1023,
    "postin": 1024,
    "preun": 1025,
    "postun": 1026,
    "filenames": 1027,
    "filesizes": 1028,
    "filestates": 1029,
    "filemodes": 1030,
    "fileuids": 1031,
    "filegids": 1032,
    "filerdevs": 1033,
    "filemtimes": 1034,
    "filemd5s": 1035,
    "filelinktos": 1036,
    "fileflags": 1037,
    "root": 1038,
    "fileusername": 1039,
    "filegroupname": 1040,
    "exclude": 1041,
    "exclusive": 1042,
    "icon": 1043,
    "sourcerpm": 1044,
    "fileverifyflags": 1045,
    "archivesize": 1046,
    "provides": 1047,
    "requireflags": 1048,
    "requirename": 1049,
    "requireversion": 1050,
    "nosource": 1051,
    "nopatch": 1052,
    "conflictflags": 1053,
    "conflictname": 1054,
    "conflictversion": 1055,
    "defaultprefix": 1056,
    "buildroot": 1057,
    "installprefix": 1058,
    "excludearch": 1059,
    "excludeos": 1060,
    "exclusivearch": 1061,
    "exclusiveos": 1062,
    "autoreqprov": 1063,
    "rpmversion": 1064,
    "triggerscripts": 1065,
    "triggername": 1066,
    "triggerversion": 1067,
    "triggerflags": 1068,
    "triggerindex": 1069,
    "verifyscript": 1079,
    "basenames": 1117,
    "archive_format": 1124,
    "archive_compression": 1125,
    "target": 1132,
    "authors": 1081,
    "comments": 1082,
    "obsoletes": 1090,
}

rtags = dict([(value, key) for (key, value) in tags.items()])


def extract_string(offset, count, store):
    assert count == 1
    idx = store[offset:].index(b"\x00")
    return store[offset : offset + idx]


def extract_array(offset, count, store):
    return store[offset:].split(b"\x00", count)[:-1]


def extract_bin(offset, count, store):
    return store[offset : offset + count]


def extract_int32(offset, count, store):
    values = struct.unpack(b"!" + b"i" * count, store[offset : offset + 4 * count])
    if count == 1:
        values = values[0]
    return values


def extract_int16(offset, count, store):
    values = struct.unpack(b"!" + b"h" * count, store[offset : offset + 2 * count])
    if count == 1:
        values = values[0]
    return values


ty_map = {
    3: extract_int16,
    4: extract_int32,
    6: extract_string,
    7: extract_bin,
    8: extract_array,
    9: extract_string,
}


def extract_data(ty, offset, count, store):
    extract = ty_map.get(ty)
    if extract:
        return extract(offset, count, store)
    else:
        return "could not extract %s" % ty


def _readheader(fileobj):
    char = fileobj.read(1)
    while char != b"\x8e":
        char = fileobj.read(1)

        if char is None or char == b"":
            raise RPMError("reached end of file without finding magic char \x8e")

    magic = b"\x8e" + fileobj.read(2)
    from binascii import hexlify

    assert hexlify(magic) == b"8eade8", hexlify(magic)
    version = ord(fileobj.read(1))

    header_start = fileobj.tell() - 4  # -4 for magic

    _ = fileobj.read(4)

    (num_entries,) = struct.unpack(b"!i", fileobj.read(4))
    (header_structure_size,) = struct.unpack(b"!i", fileobj.read(4))

    header = struct.Struct(b"!iiii")

    entries = []
    for _ in range(num_entries):
        entry = header.unpack(fileobj.read(header.size))
        entries.append(entry)

    store = fileobj.read(header_structure_size)
    store

    headers = {}
    for tag, ty, offset, count in entries:
        key = rtags.get(tag, tag)
        value = extract_data(ty, offset, count, store)
        headers[key] = value
    header_end = fileobj.tell()
    return (header_start, header_end), headers


def get_headers(fileobj):
    lead = struct.Struct(b"!4sBBhh66shh16s")
    data = fileobj.read(lead.size)
    value = lead.unpack(data)

    # Not sure what the first set of headers are for
    first_range, first_headers = _readheader(fileobj)
    second_range, second_headers = _readheader(fileobj)

    first_headers.update(second_headers)

    return second_range, first_headers


def main():

    with open(sys.argv[1]) as fileobj:
        headers = get_headers(fileobj)
        pprint(headers)


if __name__ == "__main__":
    main()



import os
import sys
import gzip
import shutil
import hashlib
import tempfile
import unittest
from functools import wraps

try:
    from urllib2 import urlopen
except ImportError:
    from urllib.request import urlopen

import rpmfile


def download(url, rpmname):
    def _downloader(func):
        @wraps(func)
        def wrapper(*args, **kwds):
            args = list(args)
            rpmpath = os.path.join(args[0].tempdir, rpmname)
            gztemp = os.path.join(args[0].tempdir, "temp.gz")
            args.append(rpmpath)
            download = urlopen(url)
            if url[::-1].startswith(".gz"[::-1]):
                with open(gztemp, "wb") as gztemp_file:
                    gztemp_file.write(download.read())
                download.close()
                download = gzip.open(gztemp, "rb")
            with open(rpmpath, "wb") as target_file:
                target_file.write(download.read())
            download.close()
            if url[::-1].startswith(".gz"[::-1]):
                os.unlink(gztemp)
            return func(*args, **kwds)

        return wrapper

    return _downloader


class TempDirTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.prevdir = os.getcwd()
        cls.tempdir = tempfile.mkdtemp()
        os.chdir(cls.tempdir)

    @classmethod
    def tearDownClass(cls):
        shutil.rmtree(cls.tempdir)
        os.chdir(cls.prevdir)

    @unittest.skipUnless(
        sys.version_info.major >= 3 and sys.version_info.minor >= 3, "Need lzma module"
    )
    @download(
        "https://download.clearlinux.org/releases/10540/clear/x86_64/os/Packages/sudo-setuid-1.8.17p1-34.x86_64.rpm",
        "sudo.rpm",
    )
    def test_lzma_sudo(self, rpmpath):
        with rpmfile.open(rpmpath) as rpm:

            # Inspect the RPM headers
            self.assertIn("name", rpm.headers.keys())
            self.assertEqual(rpm.headers.get("arch", "noarch"), b"x86_64")

            members = list(rpm.getmembers())
            self.assertEqual(len(members), 1)

            with rpm.extractfile("./usr/bin/sudo") as fd:
                calculated = hashlib.md5(fd.read()).hexdigest()
                self.assertEqual(calculated, "a208f3d9170ecfa69a0f4ccc78d2f8f6")

    @unittest.skipUnless(
        sys.version_info.major >= 3 and sys.version_info.minor >= 5, "Need io.BytesIO"
    )
    @download(
        "https://github.com/srossross/rpmfile/files/4505148/xmlstarlet-1.6.1-14.fc32.x86_64.txt",
        "xmlstarlet.rpm",
    )
    def test_zstd_xmlstarlet(self, rpmpath):
        with rpmfile.open(rpmpath) as rpm:

            # Inspect the RPM headers
            self.assertIn("name", rpm.headers.keys())
            self.assertEqual(rpm.headers.get("arch", "noarch"), b"x86_64")

            members = list(rpm.getmembers())
            self.assertEqual(len(members), 15)

            with rpm.extractfile("./usr/bin/xmlstarlet") as fd:
                calculated = hashlib.md5(fd.read()).hexdigest()
                self.assertEqual(calculated, "c5e22d7e47751565b56e507cb6ee375e")

            with rpm.extractfile("./usr/share/doc/xmlstarlet/ChangeLog") as fd:
                calculated = hashlib.md5(fd.read()).hexdigest()
                self.assertEqual(calculated, "68b329da9893e34099c7d8ad5cb9c940")

    @download(
        "https://download.fedoraproject.org/pub/fedora/linux/releases/30/Everything/source/tree/Packages/r/rpm-4.14.2.1-4.fc30.1.src.rpm",
        "sample.rpm",
    )
    def test_autoclose(self, rpmpath):
        """Test that RPMFile.open context manager properly closes rpm file"""

        rpm_ref = None
        with rpmfile.open(rpmpath) as rpm:
            rpm_ref = rpm

            # Inspect the RPM headers
            self.assertIn("name", rpm.headers.keys())
            self.assertEqual(rpm.headers.get("arch", "noarch"), b"x86_64")

            members = list(rpm.getmembers())
            self.assertEqual(len(members), 13)

            # Test that subfile does not close parent file by calling close and
            # then extractfile again
            fd = rpm.extractfile("rpm-4.13.90-ldflags.patch")
            calculated = hashlib.md5(fd.read()).hexdigest()
            self.assertEqual(calculated, "4841553ad9276fffd83df33643839a57")
            fd.close()

            fd = rpm.extractfile("rpm.spec")
            calculated = hashlib.md5(fd.read()).hexdigest()
            self.assertEqual(calculated, "ed96e23cf7f8dd17c459e0dd4618888c")
            fd.close()

        # Test that RPMFile owned file descriptor and that underlying file is really closed
        self.assertTrue(rpm_ref._fileobj.closed)
        self.assertTrue(rpm_ref._ownes_fd)

    @download(
        "https://github.com/srossross/rpmfile/files/3150016/gopacket-license.noarch.rpm.gz",
        "gopacket.rpm",
    )
    def test_issue_19(self, rpmpath):
        with rpmfile.open(rpmpath) as rpm:
            self.assertEqual(len(list(rpm.getmembers())), 2)



# -*- coding: iso-8859-15 -*-
# -*- Mode: Python; py-ident-offset: 4 -*-
# vim:ts=4:sw=4:et
"""
rpm definitions

Mario Morgado (BSD) https://github.com/mjvm/pyrpm
"""
__revision__ = "$Rev$"[6:-2]

RPM_LEAD_MAGIC_NUMBER = "\xed\xab\xee\xdb"
RPM_HEADER_MAGIC_NUMBER = "\x8e\xad\xe8"

RPMTAG_MIN_NUMBER = 1000
RPMTAG_MAX_NUMBER = 1146

# signature tags
RPMSIGTAG_SIZE = 1000
RPMSIGTAG_LEMD5_1 = 1001
RPMSIGTAG_PGP = 1002
RPMSIGTAG_LEMD5_2 = 1003
RPMSIGTAG_MD5 = 1004
RPMSIGTAG_GPG = 1005
RPMSIGTAG_PGP5 = 1006


MD5_SIZE = 16  # 16 bytes long
PGP_SIZE = 152  # 152 bytes long


# data types definition
RPM_DATA_TYPE_NULL = 0
RPM_DATA_TYPE_CHAR = 1
RPM_DATA_TYPE_INT8 = 2
RPM_DATA_TYPE_INT16 = 3
RPM_DATA_TYPE_INT32 = 4
RPM_DATA_TYPE_INT64 = 5
RPM_DATA_TYPE_STRING = 6
RPM_DATA_TYPE_BIN = 7
RPM_DATA_TYPE_STRING_ARRAY = 8
RPM_DATA_TYPE_I18NSTRING_TYPE = 9

RPM_DATA_TYPES = (
    RPM_DATA_TYPE_NULL,
    RPM_DATA_TYPE_CHAR,
    RPM_DATA_TYPE_INT8,
    RPM_DATA_TYPE_INT16,
    RPM_DATA_TYPE_INT32,
    RPM_DATA_TYPE_INT64,
    RPM_DATA_TYPE_STRING,
    RPM_DATA_TYPE_BIN,
    RPM_DATA_TYPE_STRING_ARRAY,
)

RPMTAG_NAME = 1000
RPMTAG_VERSION = 1001
RPMTAG_RELEASE = 1002
RPMTAG_DESCRIPTION = 1005
RPMTAG_COPYRIGHT = 1014
RPMTAG_URL = 1020
RPMTAG_ARCH = 1022


RPMTAGS = (
    RPMTAG_NAME,
    RPMTAG_VERSION,
    RPMTAG_RELEASE,
    RPMTAG_DESCRIPTION,
    RPMTAG_COPYRIGHT,
    RPMTAG_URL,
    RPMTAG_ARCH,
)



"""
@author: sean
"""

from setuptools import setup, find_packages
from os.path import isfile

if isfile("README.md"):
    with open("README.md") as readme:
        long_description = readme.read()
else:
    long_description = "???"

setup(
    name="rpmfile",
    description="Read rpm archive files",
    version="1.0.2",
    author="Sean Ross-Ross",
    author_email="srossross@gmail.com",
    url="https://github.com/srossross/rpmfile",
    license="MIT",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    tests_require=["zstandard>=0.13.0"],
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
)
